@@ -9,6 +9,12 @@ package org.mule.module.cxf;
 import static org.mule.module.cxf.HttpRequestPropertyManager.getBasePath;
 import static org.mule.module.cxf.HttpRequestPropertyManager.getRequestPath;
 import static org.mule.module.cxf.HttpRequestPropertyManager.getScheme;
+import static org.mule.transport.http.HttpConnector.HTTP_STATUS_PROPERTY;
+import static org.mule.transport.http.HttpConstants.HEADER_CONTENT_TYPE;
+import static org.mule.transport.http.HttpConstants.SC_ACCEPTED;
+import static org.mule.transport.http.HttpConstants.SC_INTERNAL_SERVER_ERROR;
+import static org.mule.module.http.api.HttpConstants.RequestProperties.HTTP_CLIENT_SSL_SESSION;
+
 import org.mule.DefaultMuleEvent;
 import org.mule.NonBlockingVoidMuleEvent;
 import org.mule.OptimizedRequestContext;
@@ -24,16 +30,20 @@ import org.mule.api.endpoint.EndpointNotFoundException;
 import org.mule.api.lifecycle.InitialisationException;
 import org.mule.api.lifecycle.Lifecycle;
 import org.mule.api.transformer.TransformerException;
+import org.mule.api.transport.NonBlockingReplyToHandler;
 import org.mule.api.transport.OutputHandler;
 import org.mule.api.transport.ReplyToHandler;
 import org.mule.config.i18n.MessageFactory;
 import org.mule.message.DefaultExceptionPayload;
 import org.mule.module.cxf.support.DelegatingOutputStream;
+import org.mule.module.cxf.transport.MuleUniversalDestination;
+import org.mule.module.http.api.HttpConstants;
+import org.mule.module.http.internal.listener.properties.SSLSessionProperties;
 import org.mule.module.xml.stax.StaxSource;
 import org.mule.processor.AbstractInterceptingMessageProcessor;
 import org.mule.transformer.types.DataTypeFactory;
+import org.mule.transport.NullPayload;
 import org.mule.transport.http.HttpConnector;
-import org.mule.transport.http.HttpConstants;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -47,6 +57,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import javax.net.ssl.SSLSession;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.transform.Source;
 import javax.xml.transform.dom.DOMSource;
@@ -63,6 +74,8 @@ import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.ExchangeImpl;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
+import org.apache.cxf.security.transport.TLSSessionInfo;
+import org.apache.cxf.service.model.BindingOperationInfo;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.staxutils.StaxUtils;
 import org.apache.cxf.transport.DestinationFactory;
@@ -100,6 +113,8 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
 
     private QueryHandler wsdlQueryHandler;
 
+    private String mimeType;
+
     @Override
     public void initialise() throws InitialisationException
     {
@@ -219,8 +234,8 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
             msg = out.toString();
         }
 
-        event.getMessage().setPayload(msg);
-        event.getMessage().setOutboundProperty(HttpConstants.HEADER_CONTENT_TYPE, ct);
+        event.getMessage().setPayload(msg, DataTypeFactory.XML_STRING);
+        event.getMessage().setOutboundProperty(HEADER_CONTENT_TYPE, ct);
         return event;
     }
 
@@ -244,7 +259,7 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
             {
                 final ReplyToHandler originalReplyToHandler = event.getReplyToHandler();
 
-                event = new DefaultMuleEvent(event, new ReplyToHandler()
+                event = new DefaultMuleEvent(event, new NonBlockingReplyToHandler()
                 {
                     @Override
                     public void processReplyTo(MuleEvent responseEvent, MuleMessage returnMessage, Object replyTo) throws MuleException
@@ -270,7 +285,7 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
                             responseEvent.getMessage().setExceptionPayload(exceptionPayload);
                             returnMessage.setOutboundProperty(HttpConnector.HTTP_STATUS_PROPERTY, 500);
                             responseEvent.setMessage(returnMessage);
-                            processExceptionReplyTo(new MessagingException(responseEvent, e), replyTo);
+                            processExceptionReplyTo(new MessagingException(responseEvent, e, CxfInboundMessageProcessor.this), replyTo);
                         }
                     }
 
@@ -305,9 +320,17 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
         final MessageImpl m = new MessageImpl();
         m.setExchange(exchange);
         final MuleMessage muleReqMsg = event.getMessage();
+        
+        if (muleReqMsg.getInboundProperty(HTTP_CLIENT_SSL_SESSION) != null)
+        {
+            SSLSessionProperties sessionProperties = muleReqMsg.getInboundProperty(HTTP_CLIENT_SSL_SESSION); 
+            SSLSession session = sessionProperties.retrieveSession();
+            TLSSessionInfo tlsinfo = new TLSSessionInfo(session.getCipherSuite(), session, session.getLocalCertificates());
+            m.put(TLSSessionInfo.class, tlsinfo);
+        }
         String method = muleReqMsg.getInboundProperty(HttpConnector.HTTP_METHOD_PROPERTY);
 
-        String ct = muleReqMsg.getInboundProperty(HttpConstants.HEADER_CONTENT_TYPE);
+        String ct = muleReqMsg.getInboundProperty(HEADER_CONTENT_TYPE);
         if (ct != null)
         {
             m.put(Message.CONTENT_TYPE, ct);
@@ -340,6 +363,24 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
         String soapAction = getSoapAction(event.getMessage());
         m.put(SoapConstants.SOAP_ACTION_PROPERTY_CAPS, soapAction);
 
+        org.apache.cxf.transport.Destination d;
+
+        if (server != null)
+        {
+            d = server.getDestination();
+        }
+        else
+        {
+            String serviceUri = getUri(event);
+
+            DestinationFactoryManager dfm = bus.getExtension(DestinationFactoryManager.class);
+            DestinationFactory df = dfm.getDestinationFactoryForUri(serviceUri);
+
+            EndpointInfo ei = new EndpointInfo();
+            ei.setAddress(serviceUri);
+            d = df.getDestination(ei);
+        }
+
         // For MULE-6829
         if (shouldSoapActionHeader())
         {
@@ -356,7 +397,7 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
             }
 
             String eventRequestUri = event.getMessageSourceURI().toString();
-            if (eventRequestUri.startsWith(JMS_TRANSPORT))
+            if (eventRequestUri.startsWith(JMS_TRANSPORT) || SoapJMSConstants.SOAP_JMS_NAMESPACE.equals(((MuleUniversalDestination) d).getEndpointInfo().getTransportId()))
             {
                 String contentType = muleReqMsg.getInboundProperty(SoapJMSConstants.CONTENTTYPE_FIELD);
                 if (contentType == null)
@@ -376,24 +417,6 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
             m.put(Message.PROTOCOL_HEADERS, protocolHeaders);
         }
 
-        org.apache.cxf.transport.Destination d;
-
-        if (server != null)
-        {
-            d = server.getDestination();
-        }
-        else
-        {
-            String serviceUri = getUri(event);
-
-            DestinationFactoryManager dfm = bus.getExtension(DestinationFactoryManager.class);
-            DestinationFactory df = dfm.getDestinationFactoryForUri(serviceUri);
-
-            EndpointInfo ei = new EndpointInfo();
-            ei.setAddress(serviceUri);
-            d = df.getDestination(ei);
-        }
-
         // Set up a listener for the response
         m.put(LocalConduit.DIRECT_DISPATCH, Boolean.TRUE);
         m.setDestination(d);
@@ -435,18 +458,34 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
         }
 
         MuleMessage muleResMsg = responseEvent.getMessage();
-        muleResMsg.setPayload(getResponseOutputHandler(exchange));
+
+        BindingOperationInfo binding = exchange.get(BindingOperationInfo.class);
+        if (null != binding && null != binding.getOperationInfo() && binding.getOperationInfo().isOneWay())
+        {
+            // For one-way operations, no envelope should be returned
+            // (http://www.w3.org/TR/soap12-part2/#http-reqbindwaitstate)
+            muleResMsg.setOutboundProperty(HTTP_STATUS_PROPERTY, SC_ACCEPTED);
+            muleResMsg.setPayload(NullPayload.getInstance());
+        }
+        else
+        {
+            muleResMsg.setPayload(getResponseOutputHandler(exchange), DataTypeFactory.XML_STRING);
+        }
 
         // Handle a fault if there is one.
         Message faultMsg = exchange.getOutFaultMessage();
         if (faultMsg != null)
         {
+            if (null != binding && null != binding.getOperationInfo() && binding.getOperationInfo().isOneWay())
+            {
+                muleResMsg.setPayload(getResponseOutputHandler(exchange), DataTypeFactory.XML_STRING);
+            }
             Exception ex = faultMsg.getContent(Exception.class);
             if (ex != null)
             {
                 ExceptionPayload exceptionPayload = new DefaultExceptionPayload(ex);
                 event.getMessage().setExceptionPayload(exceptionPayload);
-                muleResMsg.setOutboundProperty(HttpConnector.HTTP_STATUS_PROPERTY, 500);
+                muleResMsg.setOutboundProperty(HTTP_STATUS_PROPERTY, SC_INTERNAL_SERVER_ERROR);
             }
         }
 
@@ -630,4 +669,13 @@ public class CxfInboundMessageProcessor extends AbstractInterceptingMessageProce
         this.wsdlQueryHandler = wsdlQueryHandler;
     }
 
+    public String getMimeType()
+    {
+        return mimeType;
+    }
+
+    public void setMimeType(String mimeType)
+    {
+        this.mimeType = mimeType;
+    }
 }
